Token

Struct Token 

pub struct Token { /* private fields */ }
A token for use in an L-system. In general, the LSystem owns the token, while the user can refer to the token via an ArenaId. This means we don’t have to deal with any tricky ownership issues.

Implementations

Source

impl Token

pub fn new<T: Into<String>>(name: T) -> Self

Create a new token with the given name.

Panics

This function will panic if name contains any spaces

pub fn name(&self) -> &String

Get the name of this token