pub struct BaseTurtle { /* private fields */ }Implementations
Sourceimpl BaseTurtle
impl BaseTurtle
Sourcepub fn lines(&self) -> &[(i32, i32, i32, i32)]
pub fn lines(&self) -> &[(i32, i32, i32, i32)]
Returns a slice containing all the lines (x1, y1, x2, y2) traversed by the turtle.
Sourcepub fn set_position(&mut self, x: i32, y: i32)
pub fn set_position(&mut self, x: i32, y: i32)
Set the current position of this turtle to (x,y).
Sourcepub fn delta_move(&mut self, dx: i32, dy: i32)
pub fn delta_move(&mut self, dx: i32, dy: i32)
Moves the turtle by (dx,dy).
Sourcepub fn bounds(&self) -> (u32, u32, i32, i32)
pub fn bounds(&self) -> (u32, u32, i32, i32)
Returns (total_width, total_height, min_x, min_y), where
total_width (respectively total_height) is the largest horizontal (respectively vertical) distance between any two points that the turtle visited, min_x(respectivelymin_y`) is the smallest horizontal (respectively vertical) position that
the turtle visited.
This is useful for converting from turtle coordinates to a new coordinate system starting at (0,0)
with width total_width, height total_height, and all positions have positive x and y coordinates.