Stack

Trait Stack 

pub trait Stack: MovingTurtle {
    // Required methods
    fn push(&mut self);
    fn pop(&mut self);
}

Required Methods

fn push(&mut self)

Push the current state of this turtle onto a stack.

fn pop(&mut self)

Pop the current state of this turtle onto a stack.

Implementors